# -*- coding: utf-8 -*-
# vi:ts=4:et
#
# $Date: 2003/10/19 07:56:59 $
# $Revision: 1.5 $
# =====================================================

"""functional tools"""

# following recipes might help
# "Python Cookbook"
# o 17.11 "Wrapping an Unbounded Iterator"
# o 17.12 "Operating on Iterators"

import itertools
import sys


__all__ = [
           'drop',
           'take',
           ]

class Iterator(object):
    """functional tools
    """
    def take(self, *args):
        """"take" provides an interface nearly identical to itertools.islice.
        A subiterator limitied to the first n items of iterable's sequence
        """

        it = itertools.islice(iter(self), *args)
        for x in it:
            print x

    def drop(self, n):
        """A subiterator removing the first n items from iterable's sequence
        """
        iterable = iter(self)
        for i in range(n):
            try:
                iterable.next()
            except StopIteration:
                return

        while True:
            try:
                #yield iterable.next()
                print iterable.next()
            except StopIteration:
                return

    def tail(self, n):
        """A subiterator limitied to the last n items of iterable's sequence
        """
        iterable = iter(self)
        num_of_drop = self.size() - n
        for i in range(num_of_drop):
            try:
                iterable.next()
            except StopIteration:
                return

        while True:
            try:
                #yield iterable.next()
                print iterable.next()
            except StopIteration:
                return

#take = itertools.islice(iterable, *args)?
def take(iterable, *args):
    """A subiterator limitied to the first n items of iterable's sequence"""

    it = itertools.islice(iterable, *args)
    while True:
        try:
            yield it.next()
        except StopIteration:
            return


#drop = itertools.islice(iterable, n, None)?
def drop(iterable, n):
    """A subiterator removing the first n items from
    iterable's sequence
    """
    it = iter(iterable)
    for i in range(n):
        try:
            it.next()
        except StopIteration:
            return

    while True:
        try:
            yield it.next()
        except StopIteration:
            return

def tail(iterable, n):
    """A subiterator limitied to the last n items of iterable's sequence
    """
    # XXX
    # You need to know the size of the iterable in advance.
    # So this function may not be practical.

    try:
        num_of_drop = len(iterable) - n
    except TypeError, e:
        raise TypeError,\
    "failed to get the size of the iterable in advance.\nuse take or drop instead."

    return drop(iterable, num_of_drop)

def _test():
    # test

    print "take"
    l = range(20)
    for x in take(l, 5):
        print x

    print "slice"
    l = range(20)
    for x in take(l, 3, 10, 2):
        print x

    print "drop"
    l = range(20)
    for x in drop(l, 15):
        print x

if __name__ == '__main__':
    _test()

# The following sets edit modes for GNU EMACS
# Local Variables:
# mode:python
# End:

